function [rej_R, rej_NR, pval, q_chosen, left_max, right_max] = rdperm(w,z,c,alpha,q,q_level,P,J)

%DESCRIPTION
%Test procedure to determine exogeneity in RDD
%VERSION: 2.0.1
%DATE:
%AUTHORS:
%OUTPUT
%--p_val is the computed p value
%--rej_R = {0,1}, 1 is rejection of null under a randomized test
%--rej_NR = {0,1}, 1 is rejection of null under a non randomized test
%--q_chosen is the number of observations used from each side of threshold
%--left_max is the implied neighbourhood from the left
%--right_max is the im plied neighbourhood from the right
%INPUT
%--w is the vector of covariates
%--z is the vector of running variable
%--c is the known value of the cutoff
%--alpha is the choice of significance at which the test is conducted
%--q is the choice of tuning parameter that indicates number of neighbours
%  -- 1 is RoT
%  -- q_level multiple the choice of q by chosen constant
%--P indicates number of permutations that are performed
%--J = {0,1} determines if CvM or Max stat is used in joint test
%  -- 0 is CvM
%  -- 1 is Max 

%SET SEED
 s                      = RandStream('mrg32k3a')                            ;
 RandStream.setGlobalStream(s)                                              ;

N_w                    = length(w)                                         ;%Sample size of w
N_z                    = length(z)                                         ;%Sample size of z

if N_w ~= N_z                                                              ;%Error message for size of w and z not equal
    error('Error. size of variables do not match')
end

N                      = N_w;
Z                      = z - c                                             ;%Normalized running variable
W                      = w                                                 ;%Placeholder for observations
K                      = size(W,2);
if q == 1
   sz    = std(Z)                                                          ;
   h     = 1.84*sz*(N^(-1/5))                                              ;
   I     = (Z >= -h & Z <= h)                                              ;
   kerf  = @(z)(abs(z) <= 1).*(1 - abs(z))                                 ;
   f     = sum(kerf(Z/h))./(N*h)                                           ;
   q     = zeros(K,1)                                                      ;
   for k=1:K
     W_temp = W(:,k)                                                       ;
     p      = corrcoef(Z,W_temp)                                           ;
     p      = p(1,2)                                                       ;
     q_temp = f*sqrt((1-p^2))*sz*(N^(0.9)/log(N))                          ; 
     q(k,1) = q_temp                                                       ;
   end
   q      = ceil(min(q))                                                   ;
   q      = (q < 10)*10 + (q > N^(0.9)/log(N))*ceil(N^(0.9)/log(N)) +...
            (q <= (N^(0.9)/log(N)) & q >= 10)*q                            ;
   q      = ceil(q_level*q)                                                ;
else
   q      = ceil(q_level*q)                                                ;
   q      = (q < 10)*10 + (q > N^(0.9)/log(N))*ceil(N^(0.9)/log(N)) +...
            (q <= (N^(0.9)/log(N)) & q >= 10)*q                            ;
end

perm_mat  = zeros((2*q),P)                                                 ;
for i=1:P
     perm_mat(:,i) = randperm((2*q)).'                                     ;
end

obs_left               = (Z < 0)                                           ;%Obs to left of the cutoff
Z_left                 = Z(obs_left)                                       ;%Z obs to left of cutoff
W_left                 = W(obs_left,:)                                     ;%W Obs to left of cutoff
n                      = sum(obs_left)                                     ;%No. of obs to left of cutoff
[~, I]                 = sort(Z_left)                                      ;%Sorting obs to left of cutoff
W_left                 = W_left(I((n-q+1):n),:)                            ;%q closest W from the left of cutoff
left_max               = Z_left(I(n-q+1))                                  ;
obs_right              = (Z >= 0)                                          ;%Obs to right of the cutoff
Z_right                = Z(obs_right)                                      ;%Z obs to right of cutoff
W_right                = W(obs_right,:)                                    ;%W Obs to right of cutoff
m                      = sum(obs_right)                                    ;%No. of obs to right of cutoff
[~, I]                 = sort(Z_right)                                     ;%Sorting obs to right of cutoff
W_right                = W_right(I(1:q),:)                                 ;%q closest W from the right of cutoff
right_max              = Z_right(I(q))                                     ;

W_q                    = [W_left ; W_right]                                ;%W obs used in permutations
k                      = ceil(P*(1-alpha))                                 ;%95th value in permutations

if q > min(n,m)                                                            ;%Error message for q is greater than min(n_1,n_0)
    error('Error. q is not smaller than min(n_1,n_0)')
end

%----Compute Test Statistic------------------------------------------------

if K >= 1 && J == 1
  C              = 100 - K                                                 ;
  c              = rand(K,C)                                               ;
  c              = c./repmat(sum(c),K,1)                                   ;
  for i=1:K
      temp       = zeros(K,1)                                              ;
      temp(i,:)  = 1                                                       ;
      c(:,C+i)   = temp                                                    ;
  end
  C              = C + K                                                   ;
  W_temp     = W_q*c                                                       ;
  W_temp     = W_temp(ceil((1:q*size(W_temp,1))/q), :)                     ;
  W_3        = reshape(W_temp, [q (C*2*q)])                                ;
  W_left_1   = reshape(repmat(W_left*c,2*q,1),[size(W_left*c,1) ...
                       2*q*size(W_left*c,2)])                              ;
  W_right_1  = reshape(repmat(W_right*c,2*q,1),[size(W_right*c,1)...
                       2*q*size(W_right*c,2)])                             ;
  A_left     = double(W_left_1 <= W_3)                                     ;
  A_right    = double(W_right_1 <= W_3)                                    ;
  m_minus    = mean(A_left,1)                                              ;
  m_plus     = mean(A_right,1)                                             ;
  CvM_stat   = mean(reshape(((m_minus - m_plus).^2)',[2*q C]),1)           ;
  CvM_TS     = max(CvM_stat)                                               ;
else
  A_left            = zeros(q,(2*q),K)                                     ;
  A_right           = zeros(q,(2*q),K)                                     ;
  for ii=1:K
    W_3             = repmat(W_q(:,ii)',q,1)                               ;  
    W_left_1        = repmat(W_left(:,ii),1,(2*q))                         ; 
    A_left(:,:,ii)  = double(W_left_1 <= W_3)                              ; 
    W_right_1       = repmat(W_right(:,ii),1,(2*q))                        ;  
    A_right(:,:,ii) = double(W_right_1 <= W_3)                             ;
  end
  A_left            = prod(A_left,3)                                       ;
  m_minus           = mean(A_left,1)                                       ;
  A_right           = prod(A_right,3)                                      ;
  m_plus            = mean(A_right,1)                                      ;
  CvM_TS          = mean((m_minus - m_plus).^2)                            ;%CvM TS in sample  
end

%----Compute Permutation of Test Statistic---------------------------------

CvM_perm    = zeros(P,1)                                                   ;

if K >= 2 && J == 1
for i=1:P
    p             = perm_mat(:,i);
    W_perm        = W_q(p,:)                                               ;%Permuted w in a given sample
    W_perm_left   = W_perm(1:q,:)                                          ;
    W_perm_right  = W_perm((q+1):(2*q),:)                                  ;
    W_left_1      = reshape(repmat(W_perm_left*c,2*q,1),...
                    [size(W_perm_left*c,1) 2*q*size(W_perm_left*c,2)])     ;
    W_right_1     = reshape(repmat(W_perm_right*c,2*q,1),...
                    [size(W_perm_right*c,1) 2*q*size(W_perm_right*c,2)])   ;
    A_left        = double(W_left_1 <= W_3)                                ;
    A_right       = double(W_right_1 <= W_3)                               ;
    m_minus       = mean(A_left,1)                                         ;
    m_plus        = mean(A_right,1)                                        ;
    CvM_stat      = mean(reshape(((m_minus - m_plus).^2)',[2*q C]),1)      ;
    CvM_perm(i,1) = max(CvM_stat)                                          ;
end 
else
for i=1:P
    p             = perm_mat(:,i);
    W_perm        = W_q(p,:)                                               ;%Permuted w in a given sample
    W_perm_left   = W_perm(1:q,:)                                          ;
    W_perm_right  = W_perm((q+1):(2*q),:)                                  ;
    A_left        = zeros(q,(2*q),K)                                       ;
    A_right       = zeros(q,(2*q),K)                                       ;
    for ii=1:K
      W_3             = repmat(W_q(:,ii)',q,1)                             ;  
      W_left_1        = repmat(W_perm_left(:,ii),1,(2*q))                  ; 
      A_left(:,:,ii)  = double(W_left_1 <= W_3)                            ; 
      W_right_1       = repmat(W_perm_right(:,ii),1,(2*q))                 ; 
      A_right(:,:,ii) = double(W_right_1 <= W_3)                           ;
    end
    A_left         = prod(A_left,3)                                        ;
    mperm_minus    = mean(A_left,1)                                        ;
    A_right        = prod(A_right,3)                                       ;
    mperm_plus     = mean(A_right,1)                                       ;
    CvM_perm(i)    = mean((mperm_minus - mperm_plus).^2)                   ;%CvM TS in each permutation
end    
end
                              
%----Performing Test-------------------------------------------------------

CvM_perm_sort          = sort(CvM_perm)                                    ;%Ordering permutations of CvM TS in each sample
CvM_perm_k             = CvM_perm_sort(k)                                  ;%Obtaining kth highest value of permutation CvM TS
Mplus_CvM              = sum((CvM_perm > CvM_perm_k))                      ;%No. of permutations strictly greater than kth value
Mzero_CvM              = sum((CvM_perm == CvM_perm_k))                     ;%No. of permutations equal to the kth value
a_CvM                  = ((P*alpha) - Mplus_CvM)/(Mzero_CvM)               ;
phi_CvM                = (CvM_TS > CvM_perm_k)*1 ...
                       + (CvM_TS < CvM_perm_k)*0 ...
                       + (CvM_TS == CvM_perm_k)*(rand <= a_CvM)            ;%CvM Test procedure for NR permutation test
phi_CvM_NR             = (CvM_TS > CvM_perm_k)*1 ...
                       + (CvM_TS < CvM_perm_k)*0 ...
                       + (CvM_TS == CvM_perm_k)*0                          ;%CvM Test procedure for NR permutation test

%----Results---------------------------------------------------------------
                   
rej_R                  = phi_CvM                                           ;
rej_NR                 = phi_CvM_NR                                        ;                                    
pval                   = mean((CvM_perm >= CvM_TS))                      ;
q_chosen               = q                                                 ;

end

